@@ -23,7 +23,8 @@ use strict;
 
 package Bugzilla::CGI;
 
-use CGI qw(-no_xhtml -oldstyle_urls :private_tempfiles :unique_headers);
+use CGI qw(-no_xhtml -oldstyle_urls :private_tempfiles :unique_headers SERVER_PUSH);
+use CGI::Util qw(rearrange);
 
 use base qw(CGI);
 
@@ -45,6 +46,9 @@ sub new {
 
     my $self = $class->SUPER::new(@args);
 
+    # Make sure our outgoing cookie list is empty on each invocation
+    $self->{Bugzilla_cookie_list} = [];
+
     # Make sure that we don't send any charset headers
     $self->charset('');
 
@@ -107,22 +111,65 @@ sub canonicalise_query {
     return join("&", @parameters);
 }
 
-# CGI.pm makes this nph, but apache doesn't like that
+# Overwrite to handle nph parameter. This should stay here until perl 5.8.1 CGI
+# has been fixed to support -nph as a parameter
+#
 sub multipart_init {
+    my($self,@p) = @_;
+    my($boundary,$nph,@other) = rearrange(['BOUNDARY','NPH'],@p);
+    $boundary = $boundary || '------- =_aaaaaaaaaa0';
+    $self->{'separator'} = "\r\n--$boundary$\r\n";
+    $self->{'final_separator'} = "\r\n--$boundary--\r\n";
+    my $type = SERVER_PUSH($boundary);
+    return $self->header(
+        -nph => 0,
+        -type => $type,
+        (map { split "=", $_, 2 } @other),
+    ) . "WARNING: YOUR BROWSER DOESN'T SUPPORT THIS SERVER-PUSH TECHNOLOGY." . $self->multipart_end;
+}
+
+# Override header so we can add the cookies in
+sub header {
     my $self = shift;
 
-    unshift(@_, '-nph' => undef);
+    # Add the cookies in if we have any
+    if (scalar(@{$self->{Bugzilla_cookie_list}})) {
+        if (scalar(@_) == 1) {
+            # if there's only one parameter, then it's a Content-Type.
+            # Since we're adding parameters we have to name it.
+            unshift(@_, '-type' => shift(@_));
+        }
+        unshift(@_, '-cookie' => $self->{Bugzilla_cookie_list});
+    }
 
-    return $self->SUPER::multipart_init(@_);
+    return $self->SUPER::header(@_);
+}
+
+# We override the entirety of multipart_start instead of falling through to
+# SUPER because the built-in one can't deal with cookies in any kind of sane
+# way.  This sub is gratuitously swiped from the real CGI.pm, but fixed so
+# it actually works (but only as much as we need it to).
+sub multipart_start {
+    my(@header);
+    my($self,@p) = @_;
+    my($type,@other) = rearrange([['TYPE','CONTENT_TYPE','CONTENT-TYPE']],@p);
+    $type = $type || 'text/html';
+    push(@header,"Content-Type: $type");
+
+    # Add the cookies in if we have any
+    if (scalar(@{$self->{Bugzilla_cookie_list}})) {
+        foreach my $cookie (@{$self->{Bugzilla_cookie_list}}) {
+            push @header, "Set-Cookie: $cookie";
+        }
+    }
+
+    my $header = join($CGI::CRLF,@header)."${CGI::CRLF}${CGI::CRLF}";
+    return $header;
 }
 
 # The various parts of Bugzilla which create cookies don't want to have to
 # pass them arround to all of the callers. Instead, store them locally here,
-# and then output as required from |headers|.
-# This is done instead of just printing the result from the script, because
-# we need to use |$r->header_out| under mod_perl (which is what CGI.pm
-# does, and we need to match, plus if we don't |print| anything, we can turn
-# off mod_perl/Apache's header parsing for a small perf gain)
+# and then output as required from |header|.
 sub send_cookie {
     my $self = shift;
 
@@ -134,9 +181,7 @@ sub send_cookie {
     # we're expiring an entry.
     require CGI::Cookie;
     my $cookie = CGI::Cookie->new(@_);
-
-    # XXX - mod_perl
-    print "Set-Cookie: $cookie\r\n";
+    push @{$self->{Bugzilla_cookie_list}}, $cookie;
 
     return;
 }
