@@ -34,6 +34,7 @@ use Bugzilla::User;
 
 use Bugzilla::Error;
 use Bugzilla::Util;
+use Bugzilla::Config;
 
 # Note!  This module requires that its caller have said "require CGI.pl" 
 # to import relevant functions from that script and its companion globals.pl.
@@ -263,12 +264,12 @@ sub normalize {
     # Check for flags whose product/component is no longer included.
     &::SendSQL("
         SELECT flags.id 
-        FROM flags, bugs LEFT OUTER JOIN flaginclusions AS i
-        ON (flags.type_id = i.type_id 
+        FROM (flags INNER JOIN bugs ON flags.bug_id = bugs.bug_id)
+          LEFT OUTER JOIN flaginclusions AS i
+            ON (flags.type_id = i.type_id
             AND (bugs.product_id = i.product_id OR i.product_id IS NULL)
             AND (bugs.component_id = i.component_id OR i.component_id IS NULL))
         WHERE flags.type_id IN ($ids)
-        AND flags.bug_id = bugs.bug_id
         AND i.type_id IS NULL
     ");
     Bugzilla::Flag::clear(&::FetchOneColumn()) while &::MoreSQLData();
