@@ -225,65 +225,85 @@ sub init {
         }
     }
 
+    my $chfieldfrom = trim(lc($params->param('chfieldfrom'))) || '';
+    my $chfieldto = trim(lc($params->param('chfieldto'))) || '';
+    $chfieldfrom = '' if ($chfieldfrom eq 'now');
+    $chfieldto = '' if ($chfieldto eq 'now');
+    my @chfield = $params->param('chfield');
+    my $chvalue = trim($params->param('chfieldvalue')) || '';
+
     # 2003-05-20: The 'changedin' field is no longer in the UI, but we continue
     # to process it because it will appear in stored queries and bookmarks.
-    my $changedin = $params->param('changedin') || '';
-    $changedin = trim($changedin);
+    my $changedin = trim($params->param('changedin')) || '';
     if ($changedin) {
         if ($changedin !~ /^[0-9]*$/) {
             ThrowUserError("illegal_changed_in_last_x_days",
                               { value => $changedin });
         }
-        push(@specialchart, ["changedin", "lessthan", $changedin + 1]);
+
+        if (!$chfieldfrom
+            && !$chfieldto
+            && scalar(@chfield) == 1
+            && $chfield[0] eq "[Bug creation]")
+        {
+            # Deal with the special case where the query is using changedin
+            # to get bugs created in the last n days by converting the value
+            # into its equivalent for the chfieldfrom parameter.
+            $chfieldfrom = "-" . ($changedin - 1) . "d";
+        }
+        else {
+            # Oh boy, the general case.  Who knows why the user included
+            # the changedin parameter, but do our best to comply.
+            push(@specialchart, ["changedin", "lessthan", $changedin + 1]);
+        }
     }
 
-    my $chfieldfrom = $params->param('chfieldfrom') || '';
-    my $chfieldto = $params->param('chfieldto') || '';
-    my @chfield = $params->param('chfield');
-    my $chvalue = $params->param('chfieldvalue') || '';
-    my $lcchfieldfrom = trim(lc($chfieldfrom));
-    my $lcchfieldto = trim(lc($chfieldto));
-    $chvalue = trim($chvalue);
-
-    $lcchfieldfrom = '' if ($lcchfieldfrom eq 'now');
-    $lcchfieldto = '' if ($lcchfieldto eq 'now');
-    if ($lcchfieldfrom ne '' || $lcchfieldto ne '') {
-        my $sql_chfrom = $lcchfieldfrom ? &::SqlQuote(SqlifyDate($lcchfieldfrom)):'';
-        my $sql_chto   = $lcchfieldto   ? &::SqlQuote(SqlifyDate($lcchfieldto))  :'';
+    if ($chfieldfrom ne '' || $chfieldto ne '') {
+        my $sql_chfrom = $chfieldfrom ? &::SqlQuote(SqlifyDate($chfieldfrom)):'';
+        my $sql_chto   = $chfieldto   ? &::SqlQuote(SqlifyDate($chfieldto))  :'';
         my $sql_chvalue = $chvalue ne '' ? &::SqlQuote($chvalue) : '';
         if(!@chfield) {
             push(@wherepart, "bugs.delta_ts >= $sql_chfrom") if ($sql_chfrom);
             push(@wherepart, "bugs.delta_ts <= $sql_chto") if ($sql_chto);
         } else {
-            push(@supptables, "bugs_activity actcheck");
-            my $sql_bugschanged = '';
+            my $bug_creation_clause;
             my @list;
             foreach my $f (@chfield) {
                 if ($f eq "[Bug creation]") {
+                    # Treat [Bug creation] differently because we need to look
+                    # at bugs.creation_ts rather than the bugs_activity table.
                     my @l;
                     push(@l, "creation_ts >= $sql_chfrom") if($sql_chfrom);
                     push(@l, "creation_ts <= $sql_chto") if($sql_chto);
-                    $sql_bugschanged = "(" . join(' AND ', @l) . ")";
+                    $bug_creation_clause = "(" . join(' AND ', @l) . ")";
                 } else {
                     push(@list, "\nactcheck.fieldid = " . &::GetFieldID($f));
                 }
             }
+
+            # @list won't have any elements if the only field being searched
+            # is [Bug creation] (in which case we don't need bugs_activity).
             if(@list) {
-                $sql_bugschanged .= ' OR ' if($sql_bugschanged ne '');
-                $sql_bugschanged .= "(actcheck.bug_id = bugs.bug_id AND " .
-                                       "(" . join(' OR ', @list) . ")";
+                push(@supptables, "bugs_activity actcheck");
+                push(@wherepart, "actcheck.bug_id = bugs.bug_id");
                 if($sql_chfrom) {
-                    $sql_bugschanged .= " AND actcheck.bug_when >= $sql_chfrom";
+                    push(@wherepart, "actcheck.bug_when >= $sql_chfrom");
                 }
                 if($sql_chto) {
-                    $sql_bugschanged .= " AND actcheck.bug_when <= $sql_chto";
+                    push(@wherepart, "actcheck.bug_when <= $sql_chto");
                 }
                 if($sql_chvalue) {
-                    $sql_bugschanged .= " AND actcheck.added = $sql_chvalue";
+                    push(@wherepart, "actcheck.added = $sql_chvalue");
                 }
-                $sql_bugschanged .= ')';
             }
-            push(@wherepart, "($sql_bugschanged)");
+
+            # Now that we're done using @list to determine if there are any
+            # regular fields to search (and thus we need bugs_activity),
+            # add the [Bug creation] criterion to the list so we can OR it
+            # together with the others.
+            push(@list, $bug_creation_clause) if $bug_creation_clause;
+
+            push(@wherepart, "(" . join(" OR ", @list) . ")");
         }
     }
 
@@ -398,7 +418,6 @@ sub init {
              {
                  push(@wherepart, "$table.isprivate < 1");
              }
-             push(@wherepart, "$table.bug_id = bugs.bug_id");
 
              # Create search terms to add to the SELECT and WHERE clauses.
              # $term1 searches comments.
@@ -674,7 +693,7 @@ sub init {
              $term = "$ff != $q";
          },
          ",casesubstring" => sub {
-             $term = "INSTR($ff, $q)";
+             $term = "INSTR(CAST($ff AS BINARY), CAST($q AS BINARY))";
          },
          ",substring" => sub {
              $term = "INSTR(LOWER($ff), " . lc($q) . ")";
