@@ -29,7 +29,7 @@ package Bugzilla::Template;
 
 use strict;
 
-use Bugzilla::Config;
+use Bugzilla::Config qw(:DEFAULT $templatedir $datadir);
 use Bugzilla::Util;
 
 # for time2str - replace by TT Date plugin??
@@ -37,6 +37,7 @@ use Date::Format ();
 
 use base qw(Template);
 
+# XXX - mod_perl
 my $template_include_path;
 
 # Make an ordered list out of a HTTP Accept-Language header see RFC 2616, 14.4
@@ -69,13 +70,17 @@ sub sortAcceptLanguage {
 # If no Accept-Language is present it uses the defined default
 sub getTemplateIncludePath () {
     # Return cached value if available
+
+    # XXXX - mod_perl!
     if ($template_include_path) {
         return $template_include_path;
     }
     my $languages = trim(Param('languages'));
     if (not ($languages =~ /,/)) {
         return $template_include_path =
-               ["template/$languages/custom", "template/$languages/default"];
+               ["$templatedir/$languages/custom",
+                "$templatedir/$languages/extension",
+                "$templatedir/$languages/default"];
     }
     my @languages       = sortAcceptLanguage($languages);
     my @accept_language = sortAcceptLanguage($ENV{'HTTP_ACCEPT_LANGUAGE'} || "" );
@@ -92,7 +97,10 @@ sub getTemplateIncludePath () {
     }
     push(@usedlanguages, Param('defaultlanguage'));
     return $template_include_path =
-        [map(("template/$_/custom", "template/$_/default"), @usedlanguages)];
+        [map(("$templatedir/$_/custom",
+              "$templatedir/$_/extension",
+              "$templatedir/$_/default"),
+             @usedlanguages)];
 }
 
 
@@ -177,7 +185,10 @@ sub create {
         PRE_CHOMP => 1,
         TRIM => 1,
 
-        COMPILE_DIR => 'data/',
+        COMPILE_DIR => "$datadir/template",
+
+        # Initialize templates (f.e. by loading plugins like Hook).
+        PRE_PROCESS => "global/initialize.none.tmpl",
 
         # Functions for processing text within templates in various ways.
         # IMPORTANT!  When adding a filter here that does not override a
@@ -247,6 +258,30 @@ sub create {
                 return $var;
             } ,
 
+            # Format a filesize in bytes to a human readable value
+            unitconvert => sub
+            {
+                my ($data) = @_;
+                my $retval = "";
+                my %units = (
+                    'KB' => 1024,
+                    'MB' => 1024 * 1024,
+                    'GB' => 1024 * 1024 * 1024,
+                );
+
+                if ($data < 1024) {
+                    return "$data bytes";
+                } 
+                else {
+                    my $u;
+                    foreach $u ('GB', 'MB', 'KB') {
+                        if ($data >= $units{$u}) {
+                            return sprintf("%.2f %s", $data/$units{$u}, $u);
+                        }
+                    }
+                }
+            },
+
             # Format a time for display (more info in Bugzilla::Util)
             time => \&Bugzilla::Util::format_time,
 
@@ -321,4 +356,3 @@ C<Bugzilla-E<gt>instance-E<gt>template> to get an already created module.
 =head1 SEE ALSO
 
 L<Bugzilla>, L<Template>
-
