@@ -38,6 +38,7 @@ use Bugzilla::Search;
 use Bugzilla::User;
 
 use Bugzilla;
+use Bugzilla::Config qw(:DEFAULT $datadir);
 
 # Turn off output buffering (probably needed when displaying output feedback
 # in the regenerate mode.)
@@ -66,7 +67,7 @@ push( @myproducts, "-All-", @::legal_product );
 
 my $tstart = time;
 foreach (@myproducts) {
-    my $dir = "data/mining";
+    my $dir = "$datadir/mining";
 
     &check_data_dir ($dir);
     
@@ -87,8 +88,8 @@ CollectSeriesData();
 # Generate a static RDF file containing the default view of the duplicates data.
 open(CGI, "GATEWAY_INTERFACE=cmdline REQUEST_METHOD=GET QUERY_STRING=ctype=rdf ./duplicates.cgi |")
   || die "can't fork duplicates.cgi: $!";
-open(RDF, ">data/duplicates.tmp")
-  || die "can't write to data/duplicates.tmp: $!";
+open(RDF, ">$datadir/duplicates.tmp")
+  || die "can't write to $datadir/duplicates.tmp: $!";
 my $headers_done = 0;
 while (<CGI>) {
   print RDF if $headers_done;
@@ -96,9 +97,9 @@ while (<CGI>) {
 }
 close CGI;
 close RDF;
-if (-s "data/duplicates.tmp") {
-    rename("data/duplicates.rdf", "data/duplicates-old.rdf");
-    rename("data/duplicates.tmp", "data/duplicates.rdf");
+if (-s "$datadir/duplicates.tmp") {
+    rename("$datadir/duplicates.rdf", "$datadir/duplicates-old.rdf");
+    rename("$datadir/duplicates.tmp", "$datadir/duplicates.rdf");
 }
 
 sub check_data_dir {
@@ -182,11 +183,11 @@ sub calculate_dupes {
     # Save % count here in a date-named file
     # so we can read it back in to do changed counters
     # First, delete it if it exists, so we don't add to the contents of an old file
-    if (my @files = <data/duplicates/dupes$today*>) {
+    if (my @files = <$datadir/duplicates/dupes$today*>) {
         unlink @files;
     }
    
-    dbmopen(%count, "data/duplicates/dupes$today", 0644) || die "Can't open DBM dupes file: $!";
+    dbmopen(%count, "$datadir/duplicates/dupes$today", 0644) || die "Can't open DBM dupes file: $!";
 
     # Create a hash with key "a bug number", value "bug which that bug is a
     # direct dupe of" - straight from the duplicates table.
@@ -440,7 +441,13 @@ sub CollectSeriesData {
 
     CleanupChartTables() if ($days_since_epoch % 7 == 0);
 
+    # We save a copy of the main $dbh and then switch to the shadow and get
+    # that one too. Remember, these may be the same.
+    Bugzilla->switch_to_main_db();
     my $dbh = Bugzilla->dbh;
+    Bugzilla->switch_to_shadow_db();
+    my $shadow_dbh = Bugzilla->dbh;
+    
     my $serieses = $dbh->selectall_hashref("SELECT series_id, query, creator " .
                       "FROM series " .
                       "WHERE frequency != 0 AND " . 
@@ -452,6 +459,12 @@ sub CollectSeriesData {
                             "(series_id, date, value) " .
                             "VALUES (?, " . $dbh->quote($today) . ", ?)");
 
+    # We delete from the table beforehand, to avoid SQL errors if people run
+    # collectstats.pl twice on the same day.
+    my $deletesth = $dbh->prepare("DELETE FROM series_data 
+                                   WHERE series_id = ? AND date = " .
+                                   $dbh->quote($today));
+                                     
     foreach my $series_id (keys %$serieses) {
         # We set up the user for Search.pm's permission checking - each series
         # runs with the permissions of its creator.
@@ -465,15 +478,17 @@ sub CollectSeriesData {
         
         # We need to count the returned rows. Without subselects, we can't
         # do this directly in the SQL for all queries. So we do it by hand.
-        my $data = $dbh->selectall_arrayref($sql);
+        my $data = $shadow_dbh->selectall_arrayref($sql);
         
         my $count = scalar(@$data) || 0;
 
+        $deletesth->execute($series_id);
         $sth->execute($series_id, $count);
     }
 }
 
 sub CleanupChartTables {
+    Bugzilla->switch_to_main_db();
     my $dbh = Bugzilla->dbh;
 
     $dbh->do("LOCK TABLES series WRITE, user_series_map AS usm READ");
@@ -493,4 +508,5 @@ sub CleanupChartTables {
     }
    
     $dbh->do("UNLOCK TABLES");
+    Bugzilla->switch_to_shadow_db();
 }
