@@ -21,6 +21,7 @@
 #                 Gregor Fischer <fischer@suse.de>
 #                 Klaas Freitag  <freitag@suse.de>
 #                 Seth Landsman  <seth@dworkin.net>
+#                 Ludovic Dubost <ludovic@pobox.com>
 ###############################################################
 # Bugzilla: Create a new bug via email
 ###############################################################
@@ -37,7 +38,7 @@
 #
 # You need to work with bug_email.pl the MIME::Parser installed.
 # 
-# $Id: bug_email.pl,v 1.16 2003/08/22 14:59:13 justdave%syndicomm.com Exp $
+# $Id: bug_email.pl,v 1.18 2004/01/20 06:03:38 justdave%syndicomm.com Exp $
 ###############################################################
 
 # 02/12/2000 (SML)
@@ -58,6 +59,16 @@
 #    any email submitted bug will be entered with a product of PENDING, if no other product is
 #    specified in the email.
 
+# 10/21/2003 (Ludovic)
+# - added $DEFAULT_VERSION, similar to product and component above
+# - added command line switches to override version, product, and component, so separate
+#   email addresses can be used for different product/component/version combinations.
+#   Example for procmail:
+#    # Feed mail to stdin of bug_email.pl
+#    :0 Ec
+#    * !^Subject: .*[Bug .*]
+#    RESULT=|(cd $BUGZILLA_HOME/contrib && ./bug_email.pl -p='Tier_3_Operations' -c='General' )
+
 # Next round of revisions :
 # - querying a bug over email
 # - appending a bug over email
@@ -69,12 +80,15 @@
 use strict;
 use MIME::Parser;
 
-chdir '..';        # this script lives in contrib
-push @INC, "contrib/.";
-push @INC, ".";
+BEGIN {
+    chdir '..';        # this script lives in contrib
+    push @INC, "contrib/.";
+    push @INC, ".";
+}
 
 require "globals.pl";
-require "BugzillaEmail.pm";
+use BugzillaEmail;
+use Bugzilla::Config qw(:DEFAULT $datadir);
 
 use lib ".";
 use lib "../";
@@ -100,6 +114,7 @@ my $Message_ID;
 # change to use default product / component functionality
 my $DEFAULT_PRODUCT = "PENDING";
 my $DEFAULT_COMPONENT = "PENDING";
+my $DEFAULT_VERSION = "unspecified";
 
 ###############################################################
 # storeAttachments
@@ -137,8 +152,8 @@ sub storeAttachments( $$ )
 		print "Error while reading attachment $decoded_file!\n";
 		next;
 	    }
-	    # print "unlinking data/mimedump-tmp/$decoded_file";
-	    # unlink "data/mimedump-tmp/$decoded_file";
+	    # print "unlinking $datadir/mimedump-tmp/$decoded_file";
+	    # unlink "$datadir/mimedump-tmp/$decoded_file";
 	} else {
 	    # data is in the scalar 
 	    $data = $decoded_file;
@@ -248,7 +263,7 @@ sub Reply( $$$$ ) {
     die "Cannot find sender-email-address" unless defined( $Sender );
     
     if( $test ) {
-	open( MAIL, ">>data/bug_email_test.log" );
+	open( MAIL, '>>', "$datadir/bug_email_test.log" );
     }
     else {
 	open( MAIL, "| /usr/sbin/sendmail -t" );
@@ -670,6 +685,22 @@ sub extractControls( $ )
 foreach( @ARGV ) {
     $restricted = 1 if ( /-r/ );
     $test = 1 if ( /-t/ );
+
+    if ( /-p=['"]?(.+)['"]?/ )
+    {
+      $DEFAULT_PRODUCT = $1;
+    }
+
+    if ( /-c=['"]?(.+)["']?/ )
+    {
+      $DEFAULT_COMPONENT = $1;
+    }
+
+    if ( /-v=['"]?(.+)["']?/ )
+    {
+      $DEFAULT_VERSION = $1;
+    }
+
 }
 
 #
@@ -697,10 +728,10 @@ my $parser = new MIME::Parser;
 
 # Create and set the output directory:
 # FIXME: There should be a $BUGZILLA_HOME variable (SML)
-(-d "data/mimedump-tmp") or mkdir "data/mimedump-tmp",0755 or die "mkdir: $!";
-(-w "data/mimedump-tmp") or die "can't write to directory";
+(-d "$datadir/mimedump-tmp") or mkdir "$datadir/mimedump-tmp",0755 or die "mkdir: $!";
+(-w "$datadir/mimedump-tmp") or die "can't write to directory";
 
-$parser->output_dir("data/mimedump-tmp");
+$parser->output_dir("$datadir/mimedump-tmp");
     
 # Read the MIME message:
 my $entity = $parser->read(\*STDIN) or die "couldn't parse MIME stream";
@@ -925,7 +956,7 @@ CheckSystem( );
 
 ### Check values ...
 # Version
-my $Version = "";
+my $Version = "$DEFAULT_VERSION";
 $Version = CheckVersion( $Control{'product'}, $Control{'version'} ) if( defined( $Control{'version'}));
 if ( $Version eq "" ) {
     my $Text = "You did not send a value for the required key \@version!\n\n";
