@@ -24,6 +24,7 @@
 #                 Jacob Steenhagen <jake@bugzilla.org>
 #                 J. Paul Reed <preed@sigkill.com>
 #                 Bradley Baetz <bbaetz@student.usyd.edu.au>
+#                 Joseph Heenan <joseph@heenan.me.uk>
 #
 
 # This file defines all the parameters that we have a GUI to edit within
@@ -49,6 +50,8 @@ use strict;
 use vars qw(@param_list);
 use File::Spec; # for find_languages
 
+use Bugzilla::Config qw(:DEFAULT $templatedir $webdotdir);
+
 # Checking functions for the various values
 # Some generic checking functions are included in Bugzilla::Config
 
@@ -98,10 +101,10 @@ sub check_webdotbase {
             return "The file path \"$value\" is not a valid executable.  Please specify the complete file path to 'dot' if you intend to generate graphs locally.";
         }
         # Check .htaccess allows access to generated images
-        if(-e "data/webdot/.htaccess") {
-            open HTACCESS, "data/webdot/.htaccess";
+        if(-e "$webdotdir/.htaccess") {
+            open HTACCESS, "$webdotdir/.htaccess";
             if(! grep(/ \\\.png\$/,<HTACCESS>)) {
-                return "Dependency graph images are not accessible.\nAssuming that you have not modified the file, delete data/webdot/.htaccess and re-run checksetup.pl to rectify.\n";
+                return "Dependency graph images are not accessible.\nAssuming that you have not modified the file, delete $webdotdir/.htaccess and re-run checksetup.pl to rectify.\n";
             }
             close HTACCESS;
         }
@@ -150,13 +153,13 @@ sub check_loginmethod {
 }
 
 sub check_languages {
-    my @languages = split /,/, trim($_);
+    my @languages = split /[,\s]+/, trim($_[0]);
     if(!scalar(@languages)) {
        return "You need to specify a language tag."
     }
     foreach my $language (@languages) {
-       if(   ! -d 'template/'.trim($language).'/custom' 
-          && ! -d 'template/'.trim($language).'/default') {
+       if(   ! -d "$templatedir/$language/custom" 
+          && ! -d "$templatedir/$language/default") {
           return "The template directory for $language does not exist";
        }
     }
@@ -165,7 +168,7 @@ sub check_languages {
 
 sub find_languages {
     my @languages = ();
-    opendir(DIR, "template") || return "Can't open 'template' directory: $!";
+    opendir(DIR, $templatedir) || return "Can't open 'template' directory: $!";
     my @langdirs = grep { /^[a-z-]+$/i } readdir(DIR);
     closedir DIR;
 
@@ -452,6 +455,24 @@ sub find_languages {
    checker => \&check_loginmethod
   },
 
+  {
+   name => 'rememberlogin',
+   desc => 'Controls management of session cookies
+           <ul>
+           <li>on - Session cookies never expire (the user has to login only
+           once per browser).</li>
+           <li>off - Session cookies last until the users session ends (the user
+             will have to login in each new browser session).</li>
+           <li>defaulton/defaultoff - Default behavior as described
+           above, but user can choose whether bugzilla will remember his
+           login or not.</li>
+           </ul>',
+   type => 's',
+   choices => ['on', 'defaulton', 'defaultoff', 'off'],
+   default => 'on',
+   checker => \&check_multi
+  },
+
   {
    name => 'mostfreqthreshold',
    desc => 'The minimum number of duplicates a bug needs to show up on the ' .
@@ -555,16 +576,16 @@ Configure bugmail: %urlbase%userprefs.cgi?tab=email
 
   {
    name => 'whinemail',
-   desc => 'The email that gets sent to anyone who has a NEW bug that '.
-           'hasn\'t been touched for more than <b>whinedays</b>.  Within ' .
-           'this text, %email% gets replaced by the offender\'s email ' .
-           'address. %userid% gets replaced by the offender\'s bugzilla ' .
-           'login (which, in most installations, is the same as the email ' .
-           ' address.) %<i>anythingelse</i>% gets replaced by the definition ' .
-           'of that parameter (as defined on this page).<p> It is a good ' .
-           'idea to make sure this message has a valid From: address, so ' .
-           'that if the mail bounces, a real person can know that there are ' .
-           'bugs assigned to an invalid address.',
+   desc => 'The email that gets sent to anyone who has a NEW or REOPENED ' .
+           'bug that hasn\'t been touched for more than <b>whinedays</b>.  ' .
+           'Within this text, %email% gets replaced by the offender\'s ' .
+           'email address. %userid% gets replaced by the offender\'s ' .
+           'bugzilla login (which, in most installations, is the same as ' .
+           'the email address.) %<i>anythingelse</i>% gets replaced by the ' .
+           'definition of that parameter (as defined on this page).<p> It ' .
+           'is a good idea to make sure this message has a valid From: ' .
+           'address, so that if the mail bounces, a real person can know '.
+           'that there are bugs assigned to an invalid address.',
    type => 'l',
    default => 'From: %maintainer%
 To: %email%
@@ -576,9 +597,9 @@ You have one or more bugs assigned to you in the Bugzilla
 bugsystem (%urlbase%) that require
 attention.
 
-All of these bugs are in the NEW state, and have not been touched
-in %whinedays% days or more.  You need to take a look at them, and 
-decide on an initial action.
+All of these bugs are in the NEW or REOPENED state, and have not
+been touched in %whinedays% days or more.  You need to take a look
+at them, and decide on an initial action.
 
 Generally, this means one of three things:
 
@@ -591,9 +612,10 @@ Generally, this means one of three things:
 (3) You decide the bug belongs to you, but you can\'t solve it this moment.
     Just use the "Accept bug" command.
 
-To get a list of all NEW bugs, you can use this URL (bookmark it if you like!):
+To get a list of all NEW/REOPENED bugs, you can use this URL (bookmark
+it if you like!):
 
-   %urlbase%buglist.cgi?bug_status=NEW&assigned_to=%userid%
+ %urlbase%buglist.cgi?bug_status=NEW&bug_status=REOPENED&assigned_to=%userid%
 
 Or, you can use the general query page, at
 %urlbase%query.cgi.
@@ -702,7 +724,7 @@ You will get this message once a day until you\'ve dealt with these bugs!
    </ul>
    The default value is a publically-accessible webdot server. If you change
    this value, make certain that the webdot server can read files from your
-   data/webdot directory. On Apache you do this by editing the .htaccess file,
+   webdot directory. On Apache you do this by editing the .htaccess file,
    for other systems the needed measures may vary. You can run checksetup.pl
    to recreate the .htaccess file if it has been lost.',
    type => 't',
@@ -717,7 +739,7 @@ You will get this message once a day until you\'ve dealt with these bugs!
            'popular value to put here is <tt>^[^@]+$</tt>, which means ' .
            '"local usernames, no @ allowed."',
    type => 't',
-   default => q:^[^@]+@[^@]+\\.[^@]+$:,
+   default => q:^[\\w\\.\\+\\-=]+@[\\w\\.\\-]+\\.[\\w\\-]+$:,
    checker => \&check_regexp
   },
 
@@ -821,6 +843,14 @@ Reason: %reason%
    default => 'this may indicate a bug in your browser.'
   },
 
+  {
+   name => 'commentoncreate',
+   desc => 'If this option is on, the user needs to enter a description ' .
+           'when entering a new bug',
+   type => 'b',
+   default => 0
+  },
+
   {
    name => 'commentonaccept',
    desc => 'If this option is on, the user needs to enter a short comment if ' .
