@@ -255,7 +255,8 @@
   'style', 
   'style_url', 
   'bgcolor', 
-  'onload', 
+  'onload',
+  'bodyattrs', 
   'h1',
   'h2',
   'h3', 
@@ -491,6 +492,21 @@
   'const.CONTROLMAPMANDATORY', 
 ],
 
+'admin/keywords/list.html.tmpl' => [
+  'keyword.id',
+  'keyword.bug_count',
+],
+
+'admin/keywords/edit.html.tmpl' => [
+  'keyword_id',
+  'bug_count',
+],
+
+'admin/keywords/confirm-delete.html.tmpl' => [
+  'keyword_id',
+  'bug_count',
+],
+
 'admin/flag-type/confirm-delete.html.tmpl' => [
   'flag_count', 
   'flag_type.id', 
